<?php

namespace Drupal\mailjet_webform_subscription\Plugin\WebformElement;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\webform\Plugin\WebformElement\WebformCompositeBase;
use Drupal\webform\WebformSubmissionInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides an 'mail_list_checkbox' element.
 *
 * @WebformElement(
 *   id = "mail_list_checkbox",
 *   label = @Translation("Mailjet Subscription"),
 *   description = @Translation("Provides a checkbox that lets the user decide whether to subscribe to the mailjet newsletter."),
 *   category = @Translation("Mailjet Webform Subscription"),
 *   multiline = TRUE,
 *   composite = TRUE,
 *   states_wrapper = TRUE,
 * )
 */
class WebformMailListCheckbox extends WebformCompositeBase {

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $user;


  /**
   * The mailjet api wrapper.
   *
   * @var \Drupal\mailjet_webform_subscription\IMailjetApiWrapperInterface
   */
  protected $mailjetApiWrapper;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) : WebformMailListCheckbox {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->user = $container->get('current_user');
    $instance->entityTypeManager = $container->get('entity_type.manager');
    $instance->mailjetApiWrapper = $container->get('mailjet_webform_subscription.mailjet_api_wrapper');

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function getPluginLabel() {
    return $this->moduleHandler->moduleExists('mail_list_checkbox') ? $this->t('Newsletter checkbox') : parent::getPluginLabel();
  }

  /**
   * {@inheritdoc}
   */
  public function prepare(array &$element, WebformSubmissionInterface|null $webform_submission = NULL) : void {
    parent::prepare($element, $webform_submission);

    $element['#theme_wrappers'] = [];
    $element['#attached']['library'][] = 'mailjet_webform_subscription/mailjet_checkbox_composite';

    // #title display defaults to invisible.
    $element += [
      '#title_display' => 'invisible',
    ];
  }

  /**
   * {@inheritdoc}
   */
  protected function defineDefaultProperties()  : array {
    $properties = [
        // Element settings.
      'title' => '',
      'help' => '',
      'required' => FALSE,
      'first_name_field' => '',
      'last_name_field' => '',
      'email_address_field' => '',
      'confirm_template_id' => '',
      'success_template_id' => '',
      'template_name_variable' => '',
      'template_redirect_variable' => '',
      'list_id' => '',
      'success_node_id' => '',
    ];
    unset($properties['multiple__header']);
    return $properties;
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) : array {

    $machineNameFieldOptions = $this->getAvailableFormFields();
    $templateOptions = $this->mailjetApiWrapper->getMailjetTemplates();
    $listOptions = $this->mailjetApiWrapper->getMailJetLists();

    $form = parent::form($form, $form_state);

    $form['validation']['required_container']['required'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Required'),
      '#help' => $this->t('Marks the checkbox as a required field.'),
    ];
    // Machine names.
    $form['machine_names'] = [
      '#type' => 'details',
      '#title' => $this->t('Form field mapping'),
      '#help' => $this->t('Used to map to the machine names that are used within the form'),
      '#open' => TRUE,
    ];
    $form['machine_names']['first_name_field'] = [
      '#type' => 'select',
      '#title' => $this->t('First name field'),
      '#description' => $this->t('Field in this form that contains the users first name'),
      '#empty_option' => $this->t('- None -'),
      '#options' => $machineNameFieldOptions,
      '#default_value' => $this->configuration['first_name_field'] ?? NULL,
    ];
    $form['machine_names']['last_name_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Last name field'),
      '#description' => $this->t('Field in this form that contains the users last name'),
      '#empty_option' => $this->t('- None -'),
      '#options' => $machineNameFieldOptions,
      '#default_value' => $this->configuration['last_name_field'] ?? NULL,
    ];
    $form['machine_names']['email_address_field'] = [
      '#type' => 'select',
      '#title' => $this->t('Email address field'),
      '#description' => $this->t('Field in this form that contains the users email address'),
      '#options' => $machineNameFieldOptions,
      '#default_value' => $this->configuration['email_address_field'] ?? NULL,
      '#required' => TRUE,
    ];

    // Template configuration.
    $form['template_configuration'] = [
      '#type' => 'details',
      '#title' => $this->t('Template configuration'),
      '#help' => $this->t('Configuration and setup of the email templates used.'),
      '#open' => TRUE,
    ];
    $form['template_configuration']['template_name_variable'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Template name variable'),
      '#description' => $this->t('The variable to hold the name in the confirmation and success template eg {{var:first_name}}'),
      '#default_value' => $this->configuration['template_name_variable'] ?? NULL,
      '#required' => FALSE,
    ];
    $form['template_configuration']['template_redirect_variable'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Template redirect variable'),
      '#description' => $this->t('The variable to hold the redirect url used in the confirmation template eg {{var:redirect_url}}'),
      '#default_value' => $this->configuration['template_redirect_variable'] ?? NULL,
      '#required' => TRUE,
    ];
    $form['template_configuration']['confirm_template_id'] = [
      '#type' => 'select',
      '#title' => $this->t('Mailjet template for confirmation email'),
      '#description' => $this->t('The ID of the mailjet template to use for email confirmation'),
      '#default_value' => $this->configuration['confirm_template_id'] ?? NULL,
      '#options' => $templateOptions,
      '#required' => TRUE,
    ];
    $form['template_configuration']['success_template_id'] = [
      '#type' => 'select',
      '#title' => $this->t('Mailjet template for success email'),
      '#description' => $this->t('The ID of the mailjet template to use for success email'),
      '#default_value' => $this->configuration['success_template_id'] ?? NULL,
      '#empty_option' => $this->t('- None -'),
      '#options' => $templateOptions,
      '#required' => FALSE,
    ];

    $form['list_id'] = [
      '#type' => 'select',
      '#title' => $this->t('Contact list'),
      '#description' => $this->t('The mailjet list ID you would like the user to subscribe to.'),
      '#default_value' => $this->configuration['list_id'] ?? NULL,
      '#options' => $listOptions,
      '#required' => TRUE,
    ];
    $form['success_node_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Success node ID'),
      '#description' => $this->t('The node to override the default node that is displayed once the user has subscribed successfully.'),
      '#default_value' => $this->configuration['success_node_id'] ?? NULL,
      '#required' => FALSE,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) : void {
    parent::submitConfigurationForm($form, $form_state);

    $config = [
      'confirm_template_id' => $form_state->getValue(['template_configuration', 'confirm_template_id']),
      'template_name_variable' => $form_state->getValue(['template_configuration', 'template_name_variable']),
      'template_redirect_variable' => $form_state->getValue(['template_configuration', 'template_redirect_variable']),
      'success_template_id' => $form_state->getValue(['template_configuration', 'success_template_id']),
      'list_id' => $form_state->getValue('list_id'),
      'success_node_id' => $form_state->getValue('success_node_id'),
      'first_name_field' => $form_state->getValue(['machine_names', 'first_name_field']),
      'last_name_field' => $form_state->getValue(['machine_names', 'last_name_field']),
      'email_address_field' => $form_state->getValue(['machine_names', 'email_address_field']),
    ];

    $this->configuration = array_merge($this->configuration, $config);
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) : void {
    if ($form_state->getValue('success_node_id')) {
      $node = $this->entityTypeManager->getStorage('node')->load(intval($form_state->getValue('success_node_id')));
      if (!$node) {
        $form_state->setErrorByName('success_node_id', 'Node does not exist');
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function preSave(array &$element, WebformSubmissionInterface $webform_submission, $update = TRUE) : void {
    parent::preSave($element, $webform_submission, $update);

    $elementMachineName = $element['#webform_key'];
    $data = $webform_submission->getData();

    if (!$webform_submission->isNew()) {
      return;
    }

    $firstName = $data[$element['#first_name_field'] ?? NULL] ?? NULL;
    $emailAddress = $data[$element['#email_address_field']] ?? NULL;
    $acceptMailList = $data[$elementMachineName]['user_confirmation'] ?? NULL;

    $templateId = intval($element['#confirm_template_id']);
    $listId = intval($element['#list_id']);

    if (!$acceptMailList) {
      return;
    }

    $token = hash('sha256', random_bytes(55));
    $data[$elementMachineName]['token'] = $token;
    $webform_submission->setData($data);

    if (in_array(NULL, [$templateId, $listId, $emailAddress])) {
      \Drupal::logger('mailjet_webform_subscription')->error(
        $this->t(
          'Incorrect or nonexistent field when adding contact. List id: @listId, First Name: @firstName, Last Name: @lastName, Email:@email',
          [
            '@listId' => $listId,
            '@firstName' => $element['#first_name_field'],
            '@lastName' => $element['#last_name_field'],
            '@email' => $element['#email_address_field'],
          ]
        )
      );
      return;
    }

    if (in_array('', [$templateId, $listId, $emailAddress])) {
      \Drupal::logger('mailjet_webform_subscription')->error(
        $this->t(
          'Empty required form field when adding contact. List id: @listId, First Name: @firstName, Email:@email',
          [
            '@listId' => $listId,
            '@firstName' => $firstName,
            '@email' => $emailAddress,
          ]
        )
      );
      return;
    }

    $variables = [
      $element['#template_redirect_variable'] => Url::fromRoute(
        'mailjet_webform_subscription.confirm_mailjet_subscription',
        [],
        [
          'query' => [
            'token' => $token,
            'caller' => $elementMachineName,
          ],
        ]
      )->setAbsolute()->toString(),
    ];

    if ($element['#template_name_variable']) {
      $variables[$element['#template_name_variable']] = $firstName ?? '';
    }

    $this->mailjetApiWrapper->sendTemplateEmail($emailAddress, $templateId, $variables);
  }

  /**
   * Get valid inputs from the form formatted for a select input.
   */
  private function getAvailableFormFields(): array {
    $result = [];
    foreach ($this->webform->getElementsDecodedAndFlattened() as $key => $value) {
      if ($value['#type'] == 'email' || $value['#type'] == 'textfield') {
        $result[$key] = $value['#title'] . " ($key)";
      }
    }

    return $result;
  }

}
